#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "crypto_api.h"
#include "my_functions.h"

static int method = 0;
static char **(*cipher_name)(void);
static u4byte *(*set_key)(const u4byte *in_key, const u4byte key_len); 
static void (*encrypt)(const u4byte *in_blk, u4byte *out_blk);
static void (*decrypt)(const u4byte *in_blk, u4byte *out_blk);

char **clear_cipher_name(void)
{
  static char *alg_name[] = { "clean", "clean.c" };
  return alg_name;
}

u4byte *clear_set_key(const u4byte in_key[], const u4byte key_len)
{
  /* Nothing to do. */ 
  return (NULL);
}

void clear_encrypt(const u4byte in_blk[4], u4byte out_blk[4])
{
  int i;
  char *t;

  my_memcpy (out_blk, in_blk, sizeof (u4byte) * 4);
  t = (char*)out_blk;
  for (i = 0 ; i < 16; i ++, t++)
    (*t) ++;

  return ;
}

void clear_decrypt(const u4byte in_blk[4], u4byte out_blk[4])
{
  int i;
  char *t;

  my_memcpy (out_blk, in_blk, sizeof (u4byte) * 4);
  t = (char*)out_blk;
  for (i = 0 ; i < 16; i ++, t++)
    (*t) --; 

  return ;
}

int set_method ( int met )
{
  switch ( met )
  { 
    case METHOD_CLEAR: /* OK */
      method = METHOD_CLEAR;
      cipher_name = clear_cipher_name;
      set_key = clear_set_key;
      encrypt = clear_encrypt;
      decrypt = clear_decrypt;
    break;
    case METHOD_CAST:	/* OK */
      method = METHOD_CAST;
      cipher_name = cast_cipher_name;
      set_key = cast_set_key;
      encrypt = cast_encrypt;
      decrypt = cast_decrypt;
    break;
    case METHOD_CRYPTON:	/* OK */
      method = METHOD_CRYPTON;
      cipher_name = crypton_cipher_name;
      set_key = crypton_set_key;
      encrypt = crypton_encrypt;
      decrypt = crypton_decrypt;
    break;
    case METHOD_DEAL:	/* OK */
      method = METHOD_DEAL;
      cipher_name = deal_cipher_name;
      set_key = deal_set_key;
      encrypt = deal_encrypt;
      decrypt = deal_decrypt;
    break;
    case METHOD_DFC:	/* OK */
      method = METHOD_DFC;
      cipher_name = dfc_cipher_name;
      set_key = dfc_set_key;
      encrypt = dfc_encrypt;
      decrypt = dfc_decrypt;
    break;
    case METHOD_E2:	/* OK */
      method = METHOD_E2;
      cipher_name = e2_cipher_name;
      set_key = e2_set_key;
      encrypt = e2_encrypt;
      decrypt = e2_decrypt;
    break;
    case METHOD_HPC:	/* OK */
      method = METHOD_HPC;
      cipher_name = hpc_cipher_name;
      set_key = hpc_set_key;
      encrypt = hpc_encrypt;
      decrypt = hpc_decrypt;
    break;
    case METHOD_MARS:	/* OK */
      method = METHOD_MARS;
      cipher_name = mars_cipher_name;
      set_key = mars_set_key;
      encrypt = mars_encrypt;
      decrypt = mars_decrypt;
    break;
    case METHOD_RC6:	/* OK */
      method = METHOD_RC6;
      cipher_name = rc6_cipher_name; 
      set_key = rc6_set_key;
      encrypt = rc6_encrypt;
      decrypt = rc6_decrypt;
    break;
#ifdef METHOD_RIJNDAEL
    case METHOD_RIJNDAEL:	/* NOT OK !!! */
      method = METHOD_RIJNDAEL;
      cipher_name = rijndael_cipher_name;
      set_key = rijndael_set_key;
      encrypt = rijndael_encrypt;
      decrypt = rijndael_decrypt;
    break;
#endif
    case METHOD_SAFER:	/* OK */
      method = METHOD_SAFER;
      cipher_name = safer_cipher_name;
      set_key = safer_set_key;
      encrypt = safer_encrypt;
      decrypt = safer_decrypt;
    break;
    case METHOD_SERPENT:	/* OK */
      method = METHOD_SERPENT;
      cipher_name = serpent_cipher_name;
      set_key = serpent_set_key;
      encrypt = serpent_encrypt;
      decrypt = serpent_decrypt;
    break;
    case METHOD_TWOFISH: 	/* OK */	
      method = METHOD_TWOFISH;
      cipher_name = twofish_cipher_name;
      set_key = twofish_set_key;
      encrypt = twofish_encrypt;
      decrypt = twofish_decrypt;
    break;
    case METHOD_IDEA:		/* OK */
      method = METHOD_IDEA;
      cipher_name = idea_cipher_name;
      set_key = idea_set_key;
      encrypt = idea_encrypt;
      decrypt = idea_decrypt;
    break;
    case METHOD_BLOWFISH:
      method = METHOD_BLOWFISH;
      cipher_name = blowfish_cipher_name;
      set_key = blowfish_set_key;
      encrypt = blowfish_encrypt;
      decrypt = blowfish_decrypt;
    break;
    default: method = -1 ; break;
  }

  return (method);
}

int get_method ( void )
{
  return (method);
}

char **get_cipher_name(void)
{
  return (cipher_name());
}

/*
 New scheme:
 I need to keep some info, like 
 * crc32 (1 u4byte)
 * len (1 u2byte)
 * no sense data (4 u4byte - the rest)
 *
 * We put that header somewhere randomly in the crypted data;
 */

typedef struct s_header {
	u4byte crc;
	u2byte len;
	u4byte pad;
} header;

/* offset on u2byte
   ** data **
   header on 3 u2byte
   ** data **
 */

unsigned char * 
clear_to_crypted(u1byte *from, u4byte from_len, u4byte *to_len)
{
	u4byte fh_l; 	/* taille data + header*/
	u4byte fhc_l;	/* taille data + header + %16 */
	u1byte *fh_s, *to_c;
	u2byte where_is;
	header head;
	int i, j;

	*to_len = 0;

	if (from_len > (65536 - sizeof (header) - sizeof(u2byte))) {
		/* Too big string */
		return NULL;
	}

	fh_s = (u1byte *)my_malloc(
	  sizeof (u1byte) * (sizeof(header) + sizeof(u2byte) + from_len));

	if (fh_s == NULL)
		return NULL;

	head.crc = crc32 (from, from_len);
	head.len = (u2byte) from_len;
	srandom(time(NULL)); 
	head.pad = random(); 

	fh_l = from_len + sizeof(header) + sizeof(u2byte);
	fhc_l = fh_l + (16 - (16 - (fh_l % 16)?fh_l % 16:16));

	where_is = (random() % (fhc_l - 2)) + 2;

	my_memcpy (fh_s, (void *)&where_is, sizeof(u2byte));
	my_memcpy (fh_s + where_is, (void *)&head, sizeof(header));

	for (j = 0, i = 2; i < (from_len + 2 + sizeof (header)); i ++, j ++) {
		if (i == where_is)
			i += sizeof (header);
		*(fh_s + i) = *(from + j);
		/* my_memset (fh_s + i, *(from + j), 1); */
	}

	/* The string may be built, now we using the crypto directive. */

	to_c = (u1byte *)my_malloc(fhc_l * sizeof (u1byte));
	
	if (to_c == NULL) {
		free (fh_s);
		return NULL;
	}

	for (i = 0; i < fhc_l; i += 4 * sizeof(u4byte)) {
		encrypt ((u4byte *)(fh_s + i), (u4byte *)(to_c + i));
	}

	*to_len = fhc_l;

	free (fh_s);
	return (to_c);
}

unsigned char *
crypted_to_clear(u1byte *from, u4byte from_len, u4byte *to_len)
{
	u1byte *fh_c;
	u1byte *f_c;
	u2byte where_is;
	header head;
	int i;
	int j;
	u4byte crc;

	*to_len = 0;

	if (from_len % 16) {
		/* hugh ?! size doesn't even match % 16 !@# */
		return NULL;
	}

	fh_c = (u1byte *)my_malloc(from_len * sizeof(u1byte));

	if (fh_c == NULL)
		return NULL;

	/* Using the decrypto directive */

	for (i = 0; i < from_len; i += 4 * sizeof(u4byte)) {
		decrypt((u4byte *)(from + i), (u4byte *)(fh_c + i));
	}

	my_memcpy (&where_is, fh_c, 2);

	/* May be where_is is not valid. */
	if (where_is >= from_len) {
		free (fh_c);
		return NULL;
	}

	my_memcpy ((void*)&head, fh_c + where_is, sizeof (header));

	/* lets have a look about len */

	if (head.len + sizeof (u2byte) + sizeof (header) > from_len) {
		/* can't be in */
		free (fh_c);
		return NULL;
	} 

	f_c = (u1byte *)my_malloc(head.len * sizeof(u1byte));

	/* Let's rebuild the string */
 
	for (i = 2, j = 0; i < from_len; i ++,j ++) {
		if (i == where_is) i+= sizeof (header);

		*(f_c + j) = *(fh_c + i); 
	}

	free (fh_c);

	/* Test CRC */

	crc = crc32 (f_c, head.len);

	if (crc == head.crc) {
		/* ok */
		*to_len = head.len;
	} else {
		free (f_c);
		f_c = NULL;
	} 

	return f_c;
}


int set_crypt_key (const unsigned char *key, const unsigned int bits)
{
  u4byte *lala = (u4byte*)key;
  printf("size: %d\n", bits);
  set_key (lala, bits);
  return (0);
}
