#ifndef HEADER_CRYPTO_API
#define HEADER_CRYPTO_API

typedef unsigned char   u1byte; /* an 8 bit unsigned character type */
typedef unsigned short  u2byte; /* a 16 bit unsigned integer type   */
typedef unsigned int    u4byte; /* a 32 bit unsigned integer type   */

typedef signed char     s1byte; /* an 8 bit signed character type   */
typedef signed short    s2byte; /* a 16 bit signed integer type     */
typedef signed int      s4byte; /* a 32 bit signed integer type     */

#define METHOD_CLEAR			0x01
#define METHOD_CAST			0x02
#define METHOD_CRYPTON			0x03
#define METHOD_DEAL			0x04
#define METHOD_DFC			0x05
#define METHOD_E2			0x06
#define METHOD_HPC			0x07
#define METHOD_MARS			0x08
#define METHOD_RC6			0x09
#define METHOD_RIJNDAEL			0x0a
#define METHOD_SAFER			0x0b
#define METHOD_SERPENT			0x0c
#define METHOD_TWOFISH			0x0d
#define METHOD_IDEA			0x0e
#define METHOD_BLOWFISH                 0x0f

/* no cypher */

char **clear_cipher_name(void);
u4byte *clear_set_key(const u4byte in_key[], const u4byte key_len);
void clear_encrypt(const u4byte in_blk[4], u4byte out_blk[4]);
void clear_decrypt(const u4byte in_blk[4], u4byte out_blk[4]);

/* cast0.c */

char **cast_cipher_name(void);
u4byte *cast_set_key(const u4byte in_key[], const u4byte key_len);
void cast_encrypt(const u4byte in_blk[4], u4byte out_blk[4]);
void cast_decrypt(const u4byte in_blk[4], u4byte out_blk[4]);


/* crypton2.c */

char **crypton_cipher_name(void);
u4byte *crypton_set_key(const u4byte in_key[], const u4byte key_len);
void crypton_encrypt(const u4byte in_blk[4], u4byte out_blk[4]);
void crypton_decrypt(const u4byte in_blk[4], u4byte out_blk[4]);


/* deal.c */

char **deal_cipher_name(void);
u4byte *deal_set_key(const u4byte in_key[], const u4byte key_len);
void deal_encrypt(const u4byte in_blk[4], u4byte out_blk[4]);
void deal_decrypt(const u4byte in_blk[4], u4byte out_blk[4]);


/* dfc2.c */

char **dfc_cipher_name(void);
u4byte *dfc_set_key(const u4byte in_key[], const u4byte key_len);
void dfc_encrypt(const u4byte in_blk[4], u4byte out_blk[4]);
void dfc_decrypt(const u4byte in_blk[4], u4byte out_blk[4]);


/* e25.c */

char **e2_cipher_name(void);
u4byte *e2_set_key(const u4byte in_key[], const u4byte key_len);
void e2_encrypt(const u4byte in_blk[4], u4byte out_blk[4]);
void e2_decrypt(const u4byte in_blk[4], u4byte out_blk[4]);


/* hpc.c */

char **hpc_cipher_name(void);
u4byte *hpc_set_key(const u4byte in_key[], const u4byte key_len);
void hpc_encrypt(const u4byte in_blk[4], u4byte out_blk[4]);
void hpc_decrypt(const u4byte in_blk[4], u4byte out_blk[4]);


/* mars6.c */

char **mars_cipher_name(void);
u4byte *mars_set_key(const u4byte in_key[], const u4byte key_len);
void mars_encrypt(const u4byte in_blk[4], u4byte out_blk[4]);
void mars_decrypt(const u4byte in_blk[4], u4byte out_blk[4]);


/* rc62.c */
char **rc6_cipher_name(void);
u4byte *rc6_set_key(const u4byte key[], const u4byte key_len);
void rc6_encrypt(const u4byte in_blk[4], u4byte out_blk[4]);
void rc6_decrypt(const u4byte in_blk[4], u4byte out_blk[4]);

/* rijndael.c */

char **rijndael_cipher_name(void);
u4byte *rijndael_set_key(const u4byte in_key[], const u4byte key_len);
void rijndael_encrypt(const u4byte in_blk[4], u4byte out_blk[4]);
void rijndael_decrypt(const u4byte in_blk[4], u4byte out_blk[4]);


/* safer.c */

char **safer_cipher_name(void);
u4byte *safer_set_key(const u4byte in_key[], const u4byte key_len);
void safer_encrypt(const u4byte in_blk[4], u4byte out_blk[4]);
void safer_decrypt(const u4byte in_blk[4], u4byte out_blk[4]);


/* serpent.c */

char **serpent_cipher_name(void);
u4byte *serpent_set_key(const u4byte in_key[], const u4byte key_len);
void serpent_encrypt(const u4byte in_blk[4], u4byte out_blk[4]);
void serpent_decrypt(const u4byte in_blk[4], u4byte out_blk[4]);


/* twofish3.c */

char **twofish_cipher_name(void);
u4byte *twofish_set_key(const u4byte in_key[], const u4byte key_len);
void twofish_encrypt(const u4byte in_blk[4], u4byte out_blk[4]);
void twofish_decrypt(const u4byte in_blk[4], u4byte out_blk[4]);

/* idea.c */
/* only in 128 bits */
char **idea_cipher_name(void);
u4byte *idea_set_key(const u4byte in_key[], const u4byte key_len);
void idea_encrypt(const u4byte in_blk[4], u4byte out_blk[4]);
void idea_decrypt(const u4byte in_blk[4], u4byte out_blk[4]);

/* blowfish.c */
char **blowfish_cipher_name(void);
u4byte *blowfish_set_key(const u4byte in_key[], const u4byte key_len);
void blowfish_encrypt(const u4byte in_blk[4], u4byte out_blk[4]);
void blowfish_decrypt(const u4byte in_blk[4], u4byte out_blk[4]);

u1byte *uu_encode (u1byte *string, int len);
u1byte *uu_decode (u1byte *string, int *len);

unsigned int crc32(const unsigned char *s, unsigned int len);

int set_method ( int met );
int get_method ( void );
char **get_cipher_name( void );
int set_crypt_key (const unsigned char *key, const unsigned int bits);
unsigned char *clear_to_crypted(u1byte *from, u4byte from_len, u4byte *to_len);
unsigned char *crypted_to_clear(u1byte *from, u4byte from_len, u4byte *to_len);

#endif
